-- local functions and variables
--##############################################################################
local folder = "/games/minobueno/style/cylinder/"

local daytime = 0
local palette = love.image.newImageData(folder .. "palette.png")
local offset_target = 0
local board_canvas = lg.newCanvas(160,160)

-- bubbles
local psystem = lg.newParticleSystem(lg.newImage(folder .. "bubble.png"), 128)
psystem:setParticleLifetime(2, 1.5)
psystem:setEmissionRate(5)
psystem:setSizeVariation(1)
psystem:setDirection(-math.pi/2)
psystem:setSpeed(25, 50)
psystem:setLinearAcceleration(0, -50, 0, -25)
psystem:setEmissionArea("normal", 16, 0)

local anim = 0
local animquad = {}
local bg_tex
-- animated background, if your hardware supports it
local bg_isanimated = false
local limits = lg.getSystemLimits()
if limits.texturesize >= 8192 then
	for i = 0,127 do
		animquad[i] = lg.newQuad((i%8)*720, floor(i/8)*480, 720, 480, 5760, 7680)
	end
	bg_tex = lg.newImage(folder .. "anim.png")
	bg_isanimated = true
else
	bg_tex = lg.newImage(folder .. "notanim.png")
	warning.push("Your GPU does not support images >= 8192px")
	warning.push("Background will not be animated")
end

local shader_cylinder = lg.newShader [[
	extern Image front;
	extern Image back;
	extern Image board;
	extern bool is_back;
	extern vec2 offset;
	vec4 effect(vec4 color, Image tex, vec2 uv, vec2 pc)
	{
		vec4 uv2;
		vec4 col;
		if (is_back) {
			uv2 = Texel(back, uv*vec2(1,1));
			col = Texel(board, mod(uv2.rg*vec2(0.5,1) + offset + vec2(0.5,0.0),1.0)) * .5;
		} else {
			uv2 = Texel(front, uv*vec2(1,1));
			col = Texel(board, mod(uv2.rg*vec2(0.5,1) + offset,1.0));
		}
		return vec4(col.rgb,uv2.a*col.a)*color;
	}
]]

-- gets correct color from pallete, including interpolation
local getDayColor = function(row)
	return colormix(
		{palette:getPixel(floor(daytime*256), row)},
		{palette:getPixel(floor(daytime*256 + 1)%255, row)},
		(daytime*256)%1
	)
end


-- configuration and code used by main.lua
--##############################################################################
style = {

conf = {

	levels = "cylinder",
	scoring = "cylinder",
	randomizer = "modern",
	lockdelay = 60,
	lockdelay_soft = 20,
	lockreset = "infinity",

	ARE = 0,
	clear_delay = 10,
	gameover_check = "oncollision",
	delete_above_field = false,
	allow_above_field = true,

	das_delay = 10,
	das_period = 3,

	softdrop_delay = 8,
	softdrop_period = 2,
	softlock = false,

	ghost = true,
	rot_system = "srs",
	kicks = "permissive",
	hold = true,
	harddrop = true,
	softdrop = true,
	spinny = false,
	smoothfall = "late",
	smoothslide = 0,
	piece_gfx = "city",
	piece_sfx = "city",
	album = "Bach",
	song = "BWV 565",
	next = true,
},

size = {
	w = 720,
	h = 480,
},

board = {
	-- board size
	w = 20,
	h = 20,
	-- board offset
	--x = 140,
	--y = 40,
	x = 0,
	y = 0,
	-- wrap board, for cylindrical mode
	wrap = true,
},
-- offset fer piece spawning
-- x, y
spawn_offset = {
	{ 3,-1}, -- I
	{ 3, 0}, -- J
	{ 3, 0}, -- L
	{ 3,-1}, -- O
	{ 3, 0}, -- S
	{ 3, 0}, -- T
	{ 3, 0}, -- Z
},
gameover_time = 4*60,

gameover = function()
	playstylesfx("gameover")
	shader_cylinder:send("front", style.tex.cracked_front)
	shader_cylinder:send("back", style.tex.cracked_back)
end,

update = function(dt, vars)
	local held = vars.held
	psystem:update(dt)
	-- 300-second long days
	daytime = (daytime+dt/300)%1
	anim = (anim + dt)%16
	offset_target = offset_target - (offset_target - held.x/style.board.w)*dt*8
end,

init = function(vars)
	shader_cylinder:send("front", style.tex.front)
	shader_cylinder:send("back", style.tex.back)
end,

draw = function(vars, canvas)
	local held = vars.held
	local board = vars.board
	local block = vars.block

	lg.setCanvas(board_canvas)
	lg.clear()

	-- draw board
	lg.push();lg.translate(style.board.x, style.board.y)
	drawpieces.board(board, block, vars.level)
	-- draw pieces
	if game.state == "playing" or game.state == "clearing" then
		drawpieces.pieces(vars, game.state)
	end
	lg.pop()


	lg.setCanvas(canvas)

	-- main background
	lg.setColor(getDayColor(0))
	if bg_isanimated then
		lg.draw(bg_tex, animquad[floor(anim*8)],0,0,0)
	else
		lg.draw(bg_tex)
	end

	-- night overlay
	lg.setColor(getDayColor(1))
	lg.draw(style.tex.night,0,0,0)

	lg.setColor(getDayColor(0))

	shader_cylinder:send("offset", {offset_target - .15,0})
	shader_cylinder:send("board", board_canvas)

	lg.setShader(shader_cylinder)
		shader_cylinder:send("is_back", true)
		lg.draw(style.tex.front, 0, 0)
	lg.setShader()

	-- fog
	lg.draw(style.tex.fog)
	-- bubbles
	lg.draw(psystem, 360, 360)

	lg.setShader(shader_cylinder)
		shader_cylinder:send("is_back", false)
		lg.draw(style.tex.back, 0, 0)
	lg.setShader()

	-- specular overlay
	lg.setBlendMode("add")
	if game.state == "gameover" then
		lg.draw(style.tex.overlay_cracked)
	else
		lg.draw(style.tex.overlay)
	end
	lg.setBlendMode("alpha")

	lg.setColor(1,1,1)

end,

}